local addonName, addonData = ...

----------------------------------------
-- options, targetlist and memberlist (order is prio)
----------------------------------------

addonData.isDebug = false

addonData.savedVariablesAvailable = false

addonData.enableChat = true

addonData.targets = {"{rt8}", "{rt7}", "{rt6}", "{rt5}", "{rt4}", "{rt3}", "{rt2}", "{rt1}", "Hunters Mark"}

addonData.dwarfPriests = { "Chimal", "Blackbeard", "Dopamind", "Knud" }
addonData.nonDwarfPriests = { "Quent", "Allvaterjan", "Ruya", "Briséis", "Nillepeter", "Demuny" }
addonData.healPriests = { "Chimal", "Quent", "Allvaterjan", "Blackbeard", "Ruya", "Briséis", "Nillepeter", "Dopamind", "Knud", "Demuny" }
addonData.shadowPriests = { "Kallisto" }
addonData.tanks = { "Gatsu", "Schmid", "Draken", "Bumba", "Dirodan" }
addonData.offtanks = { "Achernar", "Plattfuss", "Peace", "Nyotha", "Artania", "Skytale", "Frommy", "Bigdickjohn", "Irageatnoobs", "Jikesch" }
addonData.warlocks = { "Dornrhödchen", "Deadly", "Horschti", "Tonz", "Sincap", "Kharadun", "Nicozealand" }
addonData.mages = { "Algrin", "Ultimage", "Ralathor", "Aristala", "Isilde", "Brexos", "Simsalabumm", "Nival", "Sifir", "Asúná" }
addonData.hunters = { "Keduh", "Jenzen", "Arwen", "Hárdý", "Skadi", "Afkippe" }
addonData.rogues = { "Alto", "Cyra", "Môrty", "Virome", "Jacboo", "Dabby", "Tôrvi", "Tiuz" }
addonData.retriPaladins = { }
addonData.healPaladins = { "Nahr", "Aneera", "Johnnylight", "Jormundgandr", "Dampfturbine", "Halbrad", "Börr", "Tontz" }
addonData.feralDruids = { "Tortur", "Keló" }
addonData.healDruids = { "Mazarin", "Nîki", "Merkurion", "Vitamix", "Zumbie", "Teeblatt", "Ebioida", "Forrést", "Arvyn" }
local supportMindControllers = { "Chimal", "Nillepeter", "Quent", "Blackbeard", "Allvaterjan"}

----------------------------------------
-- collection helpers
----------------------------------------

local function mergeArray(t1, t2)
    for _,v in ipairs(t2) do
        table.insert(t1, v)
    end
    return t1
end

addonData.mergeArray = mergeArray

local function indexOf(t, val)
    for k,v in ipairs(t) do 
        if v == val then
            return k
        end
    end
end

addonData.indexOf = indexOf

local function removeValue(t, val)
    local index = indexOf(t, val)
    if not index then
        return false
    end
    
    table.remove(t, index)
    return true
end

addonData.removeValue = removeValue

local function toSet(tab)
    local result = {}
    
    for index, value in ipairs(tab) do
        result[value] = true
    end
    return result
end

addonData.toSet = toSet

local function shallowcopy(orig)
    local orig_type = type(orig)
    local copy
    if orig_type == 'table' then
        copy = {}
        for orig_key, orig_value in pairs(orig) do
            copy[orig_key] = orig_value
        end
    else -- number, string, boolean, etc
        copy = orig
    end
    return copy
end

addonData.shallowcopy = shallowcopy

--------------------------
-- Memberlist processing
--------------------------

addonData.priests = {}
mergeArray(addonData.priests, addonData.healPriests)
mergeArray(addonData.priests, addonData.shadowPriests)

addonData.warriors = {}
mergeArray(addonData.warriors, addonData.tanks)
mergeArray(addonData.warriors, addonData.offtanks)

local numtanks = #addonData.tanks
mergeArray(addonData.tanks, addonData.offtanks)
for i=1, #addonData.feralDruids do
    table.insert(addonData.tanks, numtanks + i + 2, addonData.feralDruids[i])
end

addonData.paladins = {}
mergeArray(addonData.paladins, addonData.retriPaladins)
mergeArray(addonData.paladins, addonData.healPaladins)

addonData.druids = {}
mergeArray(addonData.druids, addonData.healDruids)
mergeArray(addonData.druids, addonData.feralDruids)

addonData.stunners = {}
mergeArray(addonData.stunners, addonData.rogues)
mergeArray(addonData.stunners, addonData.paladins)

addonData.interrupters = {} --alternating
local count = max(#addonData.rogues, #addonData.offtanks)
for i = 1, count do
	if addonData.rogues[i] then
		addonData.interrupters[#addonData.interrupters +1] = addonData.rogues[i]
	end
	
	if addonData.offtanks[i] then
		addonData.interrupters[#addonData.interrupters +1] = addonData.offtanks[i]
	end
end

addonData.healersAlternating = {}
local count = max(#addonData.healPaladins, #addonData.healPriests, #addonData.healDruids)
for i = 1, count do
	if addonData.healPaladins[i] then
		addonData.healersAlternating[#addonData.healersAlternating +1] = addonData.healPaladins[i]
	end
	
	if addonData.healPriests[i] then
		addonData.healersAlternating[#addonData.healersAlternating +1] = addonData.healPriests[i]
	end
    
    if addonData.healDruids[i] then
		addonData.healersAlternating[#addonData.healersAlternating +1] = addonData.healDruids[i]
	end
end

addonData.healers = {}
mergeArray(addonData.healers, addonData.healPaladins)
mergeArray(addonData.healers, addonData.healPriests)
mergeArray(addonData.healers, addonData.healDruids)
mergeArray(addonData.healers, addonData.shadowPriests)

addonData.healersPriestPrio = {}
mergeArray(addonData.healersPriestPrio, addonData.healPriests)
mergeArray(addonData.healersPriestPrio, addonData.healDruids)
mergeArray(addonData.healersPriestPrio, addonData.healPaladins)
mergeArray(addonData.healersPriestPrio, addonData.shadowPriests)

addonData.healersDruidPrio = {}
mergeArray(addonData.healersDruidPrio, addonData.healDruids)
mergeArray(addonData.healersDruidPrio, addonData.healPriests)
mergeArray(addonData.healersDruidPrio, addonData.healPaladins)
mergeArray(addonData.healersDruidPrio, addonData.shadowPriests)

addonData.healersPaladinPrio = {}
mergeArray(addonData.healersPaladinPrio, addonData.healPaladins)
mergeArray(addonData.healersPaladinPrio, addonData.healPriests)
mergeArray(addonData.healersPaladinPrio, addonData.healDruids)
mergeArray(addonData.healersPaladinPrio, addonData.shadowPriests)

addonData.mindControllers = {}
mergeArray(addonData.mindControllers, addonData.shadowPriests)
mergeArray(addonData.mindControllers, supportMindControllers)

----------------------
-- Chat and Exorsus helpers
----------------------

local chatBuffer = {}

local function sendWhisperMessage(receiver, text)
    SendChatMessage(text, "WHISPER", nil, receiver)
end

addonData.sendWhisperMessage = sendWhisperMessage

local function sendChatMessage(text)
    if addonData.enableChat then
        if not UnitInRaid("player") then
            print(text)
        else
            SendChatMessage(text, "RAID")
        end
    end
    
    chatBuffer[#chatBuffer + 1] = addonData.colorizeTextByRole(text)
end

addonData.sendChatMessage = sendChatMessage

local function sendRaidWarning(text)
    SendChatMessage(text, "RAID_WARNING")
end

addonData.sendRaidWarning = sendRaidWarning

local function getExRTNoteIndex(name)
    local index = 0
    for i=1,#VExRT.Note.Black do
        if VExRT.Note.BlackNames[i] == name then
            index = i
            break
        end
    end
    return index
end

local function setExRTNote(name, text)
    local index = getExRTNoteIndex(name)    
    if index == 0 then
        index = #VExRT.Note.Black + 1
    end
    
    VExRT.Note.Black[index] = text
    VExRT.Note.BlackNames[index] = name
    VExRT.Note.BlackLastUpdateTime[index] = time()
end

addonData.setExRTNote = setExRTNote

local function setExRTNoteActive(name)
    local index = getExRTNoteIndex(name)
    if index == 0 then
        return
    end
    
    GExRT.A["Note"].frame:Save(index)
end

addonData.setExRTNoteActive = setExRTNoteActive

local function getExRTNote(name)
    for i=1,#VExRT.Note.Black do
        if VExRT.Note.BlackNames[i] == name then
            return VExRT.Note.Black[i]
        end
    end

    return ""
end

addonData.getExRTNote = getExRTNote

local function flushChatBuffer()
    local str = table.concat(chatBuffer, "\n")
    chatBuffer = {}
    return str
end

addonData.flushChatBuffer = flushChatBuffer

------------------------------
-- Collection helpers
------------------------------

local function shallowCopy(orig)
    local orig_type = type(orig)
    local copy
    if orig_type == 'table' then
        copy = {}
        for orig_key, orig_value in pairs(orig) do
            copy[orig_key] = orig_value
        end
    else -- number, string, boolean, etc
        copy = orig
    end
    return copy
end

addonData.shallowCopy = shallowCopy

local function hasValue(tab, val)
    for index, value in ipairs(tab) do
        if value == val then
            return true
        end
    end

    return false
end

addonData.hasValue = hasValue

local function removeValue(tab, val)
    for index, value in ipairs(tab) do
        if value == val then
            table.remove(tab, index)
            return true
        end
    end

    return false
end

addonData.removeValue = removeValue

local function reverseTable(tab)
  local n = #tab
  local i = 1
  while i < n do
    tab[i],tab[n] = tab[n],tab[i]
    i = i + 1
    n = n - 1
  end
end

addonData.reverseTable = reverseTable

function pairsByKeys(t, f)
    local a = {}
    for n in pairs(t) do
        table.insert(a, n)
    end
    table.sort(a, f)
    local i = 0      -- iterator variable
    local iter = function ()   -- iterator function
        i = i + 1
        if a[i] == nil then
            return nil
        else
            return a[i], t[a[i]]
        end
    end
    return iter
end

addonData.pairsByKeys = pairsByKeys

local function copyAndRemoveNotInRaid(raidMembers, memberlist)
    local copy = {}
	
    if addonData.isDebug then
        copy = addonData.shallowCopy(memberlist)
        return copy
    end
    
    for i,member in ipairs(memberlist) do
        for i,raidMember in ipairs(raidMembers) do
            if raidMember.name == member then
                copy[#copy+1] = member
                break
            end
        end
    end

    return copy
end

addonData.copyAndRemoveNotInRaid = copyAndRemoveNotInRaid

local function getRaidMembersByRole()
    local raidMembers = addonData.getRaidMembers()
    local members = {}
    
    members.dwarfPriests = addonData.copyAndRemoveNotInRaid(raidMembers, addonData.dwarfPriests)
    members.nonDwarfPriests = addonData.copyAndRemoveNotInRaid(raidMembers, addonData.nonDwarfPriests)
    members.healPriests = addonData.copyAndRemoveNotInRaid(raidMembers, addonData.healPriests)
    members.shadowPriests = addonData.copyAndRemoveNotInRaid(raidMembers, addonData.shadowPriests)
    members.mindControllers = addonData.copyAndRemoveNotInRaid(raidMembers, addonData.mindControllers)
    members.priests = addonData.copyAndRemoveNotInRaid(raidMembers, addonData.priests)
    
    members.tanks = addonData.copyAndRemoveNotInRaid(raidMembers, addonData.tanks)
    members.offtanks = addonData.copyAndRemoveNotInRaid(raidMembers, addonData.offtanks)
    members.warriors = addonData.copyAndRemoveNotInRaid(raidMembers, addonData.warriors)
    
    members.warlocks = addonData.copyAndRemoveNotInRaid(raidMembers, addonData.warlocks)
    
    members.mages = addonData.copyAndRemoveNotInRaid(raidMembers, addonData.mages)
    
    members.hunters = addonData.copyAndRemoveNotInRaid(raidMembers, addonData.hunters)
    
    members.rogues = addonData.copyAndRemoveNotInRaid(raidMembers, addonData.rogues)
    
    members.retriPaladins = addonData.copyAndRemoveNotInRaid(raidMembers, addonData.retriPaladins)
    members.healPaladins = addonData.copyAndRemoveNotInRaid(raidMembers, addonData.healPaladins)
    members.paladins = addonData.copyAndRemoveNotInRaid(raidMembers, addonData.paladins)
    
    members.feralDruids = addonData.copyAndRemoveNotInRaid(raidMembers, addonData.feralDruids)
    members.healDruids = addonData.copyAndRemoveNotInRaid(raidMembers, addonData.healDruids)
    members.druids = addonData.copyAndRemoveNotInRaid(raidMembers, addonData.druids)
    
    members.healers = addonData.copyAndRemoveNotInRaid(raidMembers, addonData.healers)
    members.healersAlternating = addonData.copyAndRemoveNotInRaid(raidMembers, addonData.healersAlternating)
    members.healersPriestPrio = addonData.copyAndRemoveNotInRaid(raidMembers, addonData.healersPriestPrio)
    members.healersDruidPrio = addonData.copyAndRemoveNotInRaid(raidMembers, addonData.healersDruidPrio)
    members.healersPaladinPrio = addonData.copyAndRemoveNotInRaid(raidMembers, addonData.healersPaladinPrio)

    members.stunners = addonData.copyAndRemoveNotInRaid(raidMembers, addonData.stunners)
    members.interrupters = addonData.copyAndRemoveNotInRaid(raidMembers, addonData.interrupters)
    return members
end

addonData.getRaidMembersByRole = getRaidMembersByRole

local function tableCount(tab)
  local count = 0
  for _ in pairs(tab) do count = count + 1 end
  return count
end

addonData.tableCount = tableCount

-----------------------
-- Game info helpers
-----------------------

local function getRaidMembers()
    local raidMembers = {}
	
	if not IsInRaid() then
		return raidMembers
	end
	
    for i=1,40 do
        local name, rank, subgroup, level, class, fileName, zone, online, isDead, role, isML, combatRole  = GetRaidRosterInfo(i)
        if name and level > 0 then
            raidMembers[#raidMembers+1] = { name = name, class = fileName }
        end
    end

    return raidMembers
end

addonData.getRaidMembers = getRaidMembers

local function getRaidLeader()
	if not IsInRaid() then
		return nil
	end

    for i=1,40 do
        local name, rank, subgroup, level, class, fileName, zone, online, isDead, role, isML, combatRole  = GetRaidRosterInfo(i)
        if name and UnitIsGroupLeader(name) then
            return name            
        end
    end
    return nil
end

addonData.getRaidLeader = getRaidLeader

local function isRaidLeader()
    local raidIndex = UnitInRaid("player")
    local _, rank = GetRaidRosterInfo(raidIndex)
    return rank == 2
end

addonData.isRaidLeader = isRaidLeader

local function isMC()
    if addonData.isDebug == "MC" then
        return true
    end

    local _, _, _, _, _, _, _, instanceID = GetInstanceInfo()
    return instanceID == 409
end

addonData.isMC = isMC

local function isBWL()
    if addonData.isDebug == "BWL" then
        return true
    end

    local _, _, _, _, _, _, _, instanceID = GetInstanceInfo()
    return instanceID == 469
end

addonData.isBWL = isBWL

local function isAQ()
    if addonData.isDebug == "AQ" then
        return true
    end

    local _, _, _, _, _, _, _, instanceID = GetInstanceInfo()
    return instanceID == 531
end

addonData.isAQ = isAQ

local function isNax()
    if addonData.isDebug == "Nax" then
        return true
    end

    local _, _, _, _, _, _, _, instanceID = GetInstanceInfo()
    return instanceID == 533
end

addonData.isNax = isNax

---------------------------
-- Class / role coloring
---------------------------

local classColors = 
{
    ["Druid"] = "FF7D0A",
    ["Hunter"] = "A9D271",
    ["Mage"] = "40C7EB",
    ["Paladin"] = "F58CBA",
    ["Priest"] = "FFFFFF",
    ["Rogue"] = "FFF569",
    ["Warlock"] = "8787ED",
    ["Warrior"] = "C79C6E",
}

local roleToColor = 
{
    ["Druid"] = classColors.Druid,
    ["Druids"] = classColors.Druid,
    ["Feral"] = classColors.Druid,
    ["Ferals"] = classColors.Druid,
    ["Cat"] = classColors.Druid,
    ["Cats"] = classColors.Druid,
    ["Bear"] = classColors.Druid,
    ["Bears"] = classColors.Druid,
    ["Hunter"] = classColors.Hunter,
    ["Hunters"] = classColors.Hunter,
    ["Mage"] = classColors.Mage,
    ["Mages"] = classColors.Mage,
    ["Paladin"] = classColors.Paladin,
    ["Paladins"] = classColors.Paladin,
    ["Retri"] = classColors.Paladin,
    ["Retris"] = classColors.Paladin,
    ["Priest"] = classColors.Priest,
    ["Priests"] = classColors.Priest,
    ["Heal-Priest"] = classColors.Priest,
    ["Heal-Priests"] = classColors.Priest,
    ["Shadow"] = classColors.Priest,
    ["Shadow-Priest"] = classColors.Priest,
    ["Shadow-Priests"] = classColors.Priest,
    ["Rogue"] = classColors.Rogue,
    ["Rogues"] = classColors.Rogue,
    ["Sword-Rogue"] = classColors.Rogue,
    ["Sword-Rogues"] = classColors.Rogue,
    ["Dagger-Rogue"] = classColors.Rogue,
    ["Dagger-Rogues"] = classColors.Rogue,
    ["Warlock"] = classColors.Warlock,
    ["Warlocks"] = classColors.Warlock,
    ["Warrior"] = classColors.Warrior,
    ["Warriors"] = classColors.Warrior,
    ["Tank"] = classColors.Warrior,
    ["Tanks"] = classColors.Warrior,
    ["Fury"] = classColors.Warrior,
    ["Furys"] = classColors.Warrior,
}

local function colorize(text, color)
    return "|cFF" .. color .. text .. "|r"
end

addonData.colorize = colorize

local priestsSet = toSet(addonData.priests)
local warriorsSet = toSet(addonData.warriors)
local warlocksSet = toSet(addonData.warlocks)
local magesSet = toSet(addonData.mages)
local huntersSet = toSet(addonData.hunters)
local roguesSet = toSet(addonData.rogues)
local paladinsSet = toSet(addonData.paladins)
local druidsSet = toSet(addonData.druids)

local function colorizeWordByRole(word)
    if roleToColor[word] then
        return colorize(word, roleToColor[word])
    elseif druidsSet[word] then
        return colorize(word, classColors.Druid)
    elseif huntersSet[word] then
        return colorize(word, classColors.Hunter)
    elseif magesSet[word] then
        return colorize(word, classColors.Mage)
    elseif paladinsSet[word] then
        return colorize(word, classColors.Paladin)
    elseif priestsSet[word] then
        return colorize(word, classColors.Priest)
    elseif roguesSet[word] then
        return colorize(word, classColors.Rogue)
    elseif warlocksSet[word] then
        return colorize(word, classColors.Warlock)
    elseif warriorsSet[word] then
        return colorize(word, classColors.Warrior)
    end
    
    return word
end

local function colorizeTextByRole(text)
    return string.gsub(text, "%a+", colorizeWordByRole)
end

addonData.colorizeTextByRole = colorizeTextByRole

local function colorizePlayer(player)
    local _, playerClass = UnitClass(player)
    local r, g, b, _ = GetClassColor(playerClass)

    return {
        r = r,
        g = g,
        b = b,
        a = 1
    }
end

addonData.colorizePlayer = colorizePlayer

local function printColor(text, color)
    if not color then
        color = "96a9eb"
    end

    print("\124cff" .. color .. "[Monochrome]: " .. text .. "\124r")
end

addonData.printColor = printColor

local function playerNameRemoveRealm(name)
    local splits = addonData.stringSplit(name, "-")
    return splits[1]
end

addonData.playerNameRemoveRealm = playerNameRemoveRealm

------------------
-- Misc stuff
------------------

local function callback(duration, callback)
    local newFrame = CreateFrame("Frame")
    newFrame:SetScript("OnUpdate", function (self, elapsed)
        duration = duration - elapsed
        if duration <= 0 then
            callback()
            newFrame:SetScript("OnUpdate", nil)
        end
    end)
end

addonData.callback = callback

local function stringSplit(inputstr, sep)
    if sep == nil then
        sep = "%s"
    end

    local t = {}

    for str in string.gmatch(inputstr, "([^"..sep.."]+)") do
        table.insert(t, str)
    end

    return t
end

addonData.stringSplit = stringSplit

local function trimWhitespace(str)
    if not str then
        return nil
    end
    return str:match( "^%s*(.-)%s*$" )
end

addonData.trimWhitespace = trimWhitespace

local function startsWith(str, start)
   return str:sub(1, #start) == start
end

addonData.startsWith = startsWith

local function getNpcId(guid)
	local _, _, _, _, _, npcId = strsplit("-", guid)
	return tonumber(npcId)
end

addonData.getNpcId = getNpcId

local bossIds =
{
	10184, -- Onyxia
	12118, -- Lucifron
	12056, -- Geddon
	12057, -- Garr
	12259, -- Gehennas
	11988, -- Golemagg
	11982, -- Magmadar
	12018, -- Domo
	11502, -- Ragnaros
	12264, -- Shazzrah
	12098, -- Sulfuron
	12017, -- Broodlord
	14020, -- Chromaggus
	14601, -- Ebonroc
	11983, -- Firemaw
	11981, -- Flamegor
	11583, -- Nefarian
	12435, -- Razorgore
	13020, -- Vael
	15516, -- Sartura
	15727, -- Cthun
	15276, -- Veklor
	15275, -- Veknilash
	15510, -- Fankriss
	15511, -- Lord Kri
	15517, -- Ouro
	15509, -- Huhuran
	15543, -- Yauj
	15263, -- Skeram
	15544, -- Vem
	15299, -- Viscidus
	15956, -- Anub'Rekhan
	15932, -- Gluth
	16060, -- Gothik
	15953, -- Faerlina
	15931, -- Grobbulus
	15936, -- Heigan
	16062, -- Mograine
	16061, -- Razuvious
	15990, -- KelThuzad
	16065, -- Blaumeux
	16011, -- Loatheb
	15952, -- Maexxna
	15954, -- Noth
	16028, -- Patchwerk
	15989, -- Sapphiron
	16063, -- Zeliek
	15928, -- Thaddius
	16064 -- Korthazz
}

local function isBoss(npcId)
	return addonData.hasValue(bossIds, npcId)
end

addonData.isBoss = isBoss