local addonName, addonData = ...

local maxNumDebuffs = 16

allTanks = "Alle Tanks"
allMages = "Alle Mages"
allWarlocks = "Alle Warlocks"
allFurys = "Alle Furys"
allHunters = "Alle Hunter"
allDruids = "Alle Druiden"
notAvailable = "Nicht zugewiesen"
allClasses = { allTanks, allMages, allWarlocks, allFurys, allHunters, allDruids, notAvailable }

local raidDebuffs = {
	--{name = "Nightfall", members = {} },
	--{name = "Annihilator", members = {}},
    --{name = "Judgement of Wisdom", members = {}, blocksMemberFor = {"Judgement of Light"} },  
    --{name = "Arkanite Dragonling", members = {} },    
    --{name = "Thunder Clap", members = addonData.offtanks },
    --{name = "Hunter T2 (Expose Weakness)", members = allHunters },
    {name = "Thunderfury 1", members = allTanks },
    {name = "Thunderfury 2", members = allTanks },
    {name = "Expose Armor", members = { "Dabby" }, blocksDebuff = "Sunder Armor" },  
    {name = "Sunder Armor", members = allFurys },     
    {name = "Improved Shadow Bolt", members = allWarlocks },
    {name = "Improved Scorch", members = allMages },
    {name = "Ignite", members = allMages },
    {name = "Gift of Arthas", members = allTanks },
    {name = "Faerie Fire", members = allDruids },
    {name = "Mindflay", members = addonData.shadowPriests },
    {name = "Shadow Weaving", members = addonData.shadowPriests },
    {name = "Curse of Recklessness", members = addonData.warlocks, blocksMemberFor = {"Curse of Elements", "Curse of Shadow", "Curse of Weakness"} },
    {name = "Curse of Elements", members = addonData.warlocks, blocksMemberFor = {"Curse of Recklessness", "Curse of Shadow", "Curse of Weakness"} },
    {name = "Curse of Shadow", members = addonData.warlocks, blocksMemberFor = {"Curse of Recklessness", "Curse of Elements", "Curse of Weakness"} },
    {name = "Shadow Word: Pain", members = addonData.shadowPriests },
    {name = "Hunter's Mark", ids = {14325}, members = allHunters },
    {name = "Demoralizing Shout", members = addonData.mergeArray({"Plattfuss"}, addonData.offtanks) }, 
    {name = "Corruption 1", ids = {25311, 11672}, members = addonData.ddWarlocks, blocksMemberFor = {"Corruption 2", "Corruption 3", "Corruption 4"} },
    {name = "Corruption 2", ids = {25311, 11672}, members = addonData.ddWarlocks, blocksMemberFor = {"Corruption 1", "Corruption 3", "Corruption 4"} },
    {name = "Corruption 3", ids = {25311, 11672}, members = addonData.ddWarlocks, blocksMemberFor = {"Corruption 1", "Corruption 2", "Corruption 4"} },
    {name = "Corruption 4", ids = {25311, 11672}, members = addonData.ddWarlocks, blocksMemberFor = {"Corruption 1", "Corruption 2", "Corruption 3"} },
    {name = "Curse of Weakness", ids = {11708}, members = addonData.warlocks, blocksMemberFor = {"Curse of Recklessness", "Curse of Elements", "Curse of Shadow"} },
    {name = "1", members = notAvailable},
    {name = "2", members = notAvailable},
    {name = "3", members = notAvailable},
    {name = "4", members = notAvailable},
    {name = "5", members = notAvailable}
}

local function getMember(members, name)
    for i,member in ipairs(members) do
        if member.name == name then
            return member
        end
    end
    
    return nil
end

local function getAssignedDebuffs()
    numDebuffs = 0

    local raidMembers = addonData.getRaidMembers()
    local instanceName = GetInstanceInfo()

    local blockedDebuffs = {}
    local blockedMembers = {}
	local function isBlocked(memberName, debuffName)
		for i,blocked in ipairs(blockedMembers) do
			if blocked[1] == memberName and addonData.hasValue(blocked[2], debuffName) then
				return true
			end
		end
	end
	
    local assignedDebuffs = {}

    for i,debuff in ipairs(raidDebuffs) do
		local debuffMembers
        local memberCount
		if addonData.hasValue(allClasses, debuff.members) then
			debuffMembers = { debuff.members }
            if debuff.members == allTanks then
                memberCount = #addonData.copyAndRemoveNotInRaid(raidMembers, addonData.tanks)
            elseif debuff.members == allMages then
                memberCount = #addonData.copyAndRemoveNotInRaid(raidMembers, addonData.mages)
            elseif debuff.members == allWarlocks then
                memberCount = #addonData.copyAndRemoveNotInRaid(raidMembers, addonData.warlocks)
            elseif debuff.members == allFurys then
                memberCount = #addonData.copyAndRemoveNotInRaid(raidMembers, addonData.offtanks)
            elseif debuff.members == allHunters then
                memberCount = #addonData.copyAndRemoveNotInRaid(raidMembers, addonData.hunters)
            elseif debuff.members == allDruids then
                memberCount = #addonData.copyAndRemoveNotInRaid(raidMembers, addonData.druids)
            else
                print("ERROR")            
            end
		else
			debuffMembers = addonData.copyAndRemoveNotInRaid(raidMembers, debuff.members)
            memberCount = #debuffMembers
		end
        
        if addonData.hasValue(blockedDebuffs, debuff.name) then
            --skip
		elseif memberCount < (debuff.minMembers or 0) then
			--skip
        elseif addonData.isMC() and debuff.excludeInstance ~= nil and addonData.hasValue(debuff.excludeInstance, "MC") then
            --skip
        elseif addonData.isBWL() and debuff.excludeInstance ~= nil and addonData.hasValue(debuff.excludeInstance, "BWL") then
            --skip
        elseif addonData.isAQ() and debuff.excludeInstance ~= nil and addonData.hasValue(debuff.excludeInstance, "AQ") then
            --skip
        else
            local isApplied = false
            for i, debuffMember in ipairs(debuffMembers) do
                if not isBlocked(debuffMember, debuff.name) then
                    assignedDebuffs[debuff.name] = debuffMember
                    isApplied = true

                    if debuff.blocksMemberFor and #debuff.blocksMemberFor > 0 then
                        blockedMembers[#blockedMembers +1] = {debuffMember, debuff.blocksMemberFor}
                    end
                    
                    if debuff.blocksDebuff then
                        blockedDebuffs[#blockedDebuffs + 1] = debuff.blocksDebuff
                    end
                    
                    break
                end
            end  
            
            if isApplied then
                numDebuffs = numDebuffs + 1
            end
            if numDebuffs >= maxNumDebuffs then
                break
            end
        end
    end
    
    return assignedDebuffs
end

local function showDebuffs()
    local assignedDebuffs = getAssignedDebuffs()
    addonData.sendChatMessage("=== Zugewiesene Debuffs ===")
    local i = 1
    for debuff, member in addonData.pairsByKeys(assignedDebuffs) do
        addonData.sendChatMessage(i .. " - " .. debuff .. " - " .. member)
        i = i + 1
    end
end

addonData.showDebuffs = showDebuffs

local function isAllowed(memberName, spellId)
	local assignedDebuffs = getAssignedDebuffs()
	
	local debuffs = assignedDebuffs[memberName]
	if not debuffs then
		return false
	end

	for i, debuff in ipairs(debuffs) do
		if debuff.ids and addonData.hasValue(debuff.ids, spellId) then
			return true
		end
	end
	
	return false
end

addonData.isAllowed = isAllowed