local addonName, addonData = ...

local function removeMemberFromAllRoles(members, member)
    addonData.removeValue(members.dwarfPriests, member)
    addonData.removeValue(members.nonDwarfPriests, member)
    addonData.removeValue(members.healPriests, member)
    addonData.removeValue(members.shadowPriests, member)
    addonData.removeValue(members.mindControllers, member)
    addonData.removeValue(members.priests, member)
    addonData.removeValue(members.tanks, member)
    addonData.removeValue(members.offtanks, member)
    addonData.removeValue(members.warriors, member)
    addonData.removeValue(members.warlocks, member)
    addonData.removeValue(members.mages, member)
    addonData.removeValue(members.hunters, member)
    addonData.removeValue(members.rogues, member)
    addonData.removeValue(members.retriPaladins, member)
    addonData.removeValue(members.healPaladins, member)
    addonData.removeValue(members.paladins, member)
    addonData.removeValue(members.feralDruids, member)
    addonData.removeValue(members.healDruids, member)
    addonData.removeValue(members.druids, member)
    addonData.removeValue(members.healers, member)
    addonData.removeValue(members.healersAlternating, member)
    addonData.removeValue(members.healersPriestPrio, member)
    addonData.removeValue(members.healersDruidPrio, member)
    addonData.removeValue(members.healersPaladinPrio, member)
    addonData.removeValue(members.stunners, member)
    addonData.removeValue(members.interrupters, member)
end

local function assignMembersInternal(members, role, numMembers, memberIndex, targetIndex, dontRemoveFromList)
    local memberCount = 0
    local assignedList = {}
    if numMembers == 0 then
        return assignedList
    end
	
    while memberCount < numMembers do
        if role[memberIndex] == nil then
            assignedList[#assignedList+1] = { target = targetIndex, name = "n/a"}
        else
            local member = role[memberIndex]
            if not dontRemoveFromList then
                removeMemberFromAllRoles(members, member)
            end
            assignedList[#assignedList+1] = { target = targetIndex, name = member}
        end
        
        if type(targetIndex) == "number" then
            targetIndex = targetIndex + 1
        end
        memberCount = memberCount +1
    end
    
    return assignedList
end

local function assignMembersInternalReverse(members, role, numMembers, memberIndex, targetIndex, dontRemoveFromList)
    addonData.reverseTable(role)
    local assignedList = assignMembersInternal(members, role, numMembers, memberIndex, targetIndex, dontRemoveFromList)
    addonData.reverseTable(role)
    return assignedList
end

local function assignMembers(members, role, numMembers, targetIndex)
    return assignMembersInternal(members, role, numMembers, 1, targetIndex, false)
end

local function assignMembersDontRemove(members, role, numMembers, targetIndex)
    return assignMembersInternal(members, role, numMembers, 1, targetIndex, true)
end

local function assignMember(members, role, targetIndex)
    return assignMembersInternal(members, role, 1, 1, targetIndex, false)[1]
end

local function assignMemberDontRemove(members, role, targetIndex)
    return assignMembersInternal(members, role, 1, 1, targetIndex, true)[1]
end

local function assignMembersReverse(members, role, numMembers, targetIndex)
    return assignMembersInternalReverse(members, role, numMembers, 1, targetIndex, false)
end

local function assignMembersDontRemoveReverse(members, role, numMembers, targetIndex)
    return assignMembersInternalReverse(members, role, numMembers, 1, targetIndex, true)
end

local function assignMemberReverse(members, role, targetIndex)
    return assignMembersInternalReverse(members, role, 1, 1, targetIndex, false)[1]
end

local function assignMemberDontRemoveReverse(members, role, targetIndex)
    return assignMembersInternalReverse(members, role, 1, 1, targetIndex, true)[1]
end

local function assignMemberIndex(members, role, memberIndex, targetIndex)
    return assignMembersInternal(members, role, 1, memberIndex, targetIndex, false)[1]
end

local function assignMemberIndexDontRemove(members, role, memberIndex, targetIndex)
    return assignMembersInternal(members, role, 1, memberIndex, targetIndex, true)[1]
end

local function assignPlaceholder(placeholderName, targetIndex)
    return {{name = placeholderName, target = targetIndex}}
end

local function skipPlayer(memberlist, playerName)
    copy = addonData.shallowCopy(memberlist)
    addonData.removeValue(copy, playerName)
    return copy
end

local function printAssigned(assignedList1, assignedList2, assignedList3, assignedList4, assignedList5)
    for i,member in ipairs(assignedList1) do
        local assigned1 = assignedList1[i]
        local assigned2 = assignedList2 and assignedList2[i]
        local assigned3 = assignedList3 and assignedList3[i]
        local assigned4 = assignedList4 and assignedList4[i]
        local assigned5 = assignedList5 and assignedList5[i]

        local message = ""
        if type(assigned1.target) == "number" then
            message = addonData.targets[assigned1.target] .. " - " .. assigned1.name
        else
            message = assigned1.target .. " - " .. assigned1.name
        end
        
        if assigned2 then
            message = message .. " - " .. assigned2.name
        end
        
        if assigned3 then
            message = message .. " - " .. assigned3.name
        end
        
        if assigned4 then
            message = message .. " - " .. assigned4.name
        end
    
        if assigned5 then
            message = message .. " - " .. assigned5.name
        end
    
        addonData.sendChatMessage(message)
    end
end

--------------------------------
-- MC Targets
--------------------------------

local function showMC(members)
    addonData.sendChatMessage("MC standard targets")
	addonData.sendChatMessage(" ")
    addonData.sendChatMessage("= TANK - HEAL 1/2 =")
    local targetIndex = 1
    printAssigned(assignMembers(members, members.tanks, 3, targetIndex), assignMembers(members, members.healers, 3, targetIndex), assignMembers(members, members.healersPriestPrio, 3, targetIndex))
    addonData.sendChatMessage(" ")
    targetIndex = 4
    addonData.sendChatMessage("Ab Garr:")
    printAssigned(assignMembers(members, members.tanks, 1, targetIndex), assignMembers(members, members.healers, 1, targetIndex))
end

local function showSulfuron(members)
	addonData.sendChatMessage("Sulfuron Herold")
	addonData.sendChatMessage(" ")
    addonData.sendChatMessage("= TANK - HEAL - INTERRUPT 1/2/3 =")

    printAssigned({assignMemberIndex(members, members.tanks, 3, "Sulfuron")}, {assignMemberIndex(members, members.healers, 3, "Sulfuron")})
    
    local targetIndex = 1 
    
    printAssigned(
        assignMembers(members, members.tanks, 4, targetIndex),
        assignMembers(members, members.healers, 4, targetIndex),
        assignMembers(members, members.interrupters, 4, targetIndex),
        assignMembers(members, members.interrupters, 4, targetIndex),
        assignMembers(members, members.mages, 4, targetIndex)
    )
end

local function showDomo(members)     
	addonData.sendChatMessage("Majordomo Executus")
	addonData.sendChatMessage(" ")
       
    addonData.sendChatMessage("= TANK - HEAL =")
    
    printAssigned({assignMemberIndex(members, members.tanks, 3, "Majordomo")}, {assignMemberIndex(members, members.healers, 3, "Majordomo")})
    
    --First add-tank
    local targetIndex = 1
    printAssigned(assignMembers(members, members.tanks, 1, targetIndex), assignMembers(members, members.healers, 1, targetIndex))
    
    --Second tank takes two last adds
    local targetIndex = 3
    local maintank = { assignMemberDontRemove(members, members.tanks, targetIndex), assignMember(members, members.tanks, targetIndex +1) }
    local maintankHealer = assignMembers(members, members.healers, 2, targetIndex)
    
    --Second add-tank
    targetIndex = 2
    printAssigned(assignMembers(members, members.tanks, 1, targetIndex), assignMembers(members, members.healers, 1, targetIndex))
    
    printAssigned(maintank, maintankHealer)
    
    addonData.sendChatMessage(" ")
    addonData.sendChatMessage("= SHEEP =")
    targetIndex = 5
    printAssigned(assignMembers(members, members.mages, 4, targetIndex))
end

--------------------------------
-- BWL Targets
--------------------------------

local function showBWL(members)
    addonData.sendChatMessage("BWL standard targets")
	addonData.sendChatMessage(" ")
    addonData.sendChatMessage("= TANK - HEAL 1/2 =")
    local targetIndex = 1
    printAssigned(assignMembers(members, members.tanks, 3, targetIndex), assignMembers(members, members.healers, 3, targetIndex), assignMembers(members, members.healersPriestPrio, 3, targetIndex))
    
    addonData.sendChatMessage(" ")
    addonData.sendChatMessage("Nur für Wyrmkin Trash")
    targetIndex = 4
    printAssigned(assignMembers(members, members.tanks, 1, targetIndex), assignMembers(members, members.healers, 1, targetIndex))
    printAssigned(assignMembers(members, members.tanks, 1, "Backup"))
end

local function showRazorgore(members)
    addonData.sendChatMessage("Razorgore")
	addonData.sendChatMessage(" ")
	
    printAssigned({assignMemberIndex(members, members.tanks, 2, "Orb")})
    addonData.sendChatMessage(" ")
    addonData.sendChatMessage("Süden/Eingang: Gruppen 2,4,6,8")
    printAssigned(assignMembers(members, members.tanks, 1, "Tank"))
    addonData.sendChatMessage(" ")
    addonData.sendChatMessage("Norden/Gegenüber: Gruppen 1,3,5,7")
    printAssigned(assignMembers(members, members.tanks, 1, "Tank"))
end

local function showTrashVael(members)
    local targetIndex = 1
    
	addonData.sendChatMessage("Trash (Vael)")

    addonData.sendChatMessage(" ")
    addonData.sendChatMessage("= TANK - HEAL 1/2 =")
    printAssigned(assignMembers(members, members.tanks, 3, targetIndex), assignMembers(members, members.healers, 3, targetIndex), assignMembers(members, members.healersPriestPrio, 3, targetIndex))
   
    addonData.sendChatMessage(" ")
    addonData.sendChatMessage("= KITE =")
    targetIndex = 4
    printAssigned(assignMembers(members, members.hunters, 2, targetIndex))
    
    addonData.sendChatMessage(" ")
    addonData.sendChatMessage("= SLEEP =")
    targetIndex = 7
    printAssigned(assignMembers(members, members.druids, 2, targetIndex)) 
end

local function showFiremaw(members)
    addonData.sendChatMessage("Firemaw")
	addonData.sendChatMessage(" ")
    
    local mtHeal1 = assignMember(members, skipPlayer(members.healers, "Aneera"), targetIndex).name
    local mtHeal2 = assignMember(members, skipPlayer(members.healers, "Aneera"), targetIndex).name
    local mtHeal3 = assignMember(members, skipPlayer(members.healers, "Aneera"), targetIndex).name
    local mtHeal4 = assignMember(members, #members.nonDwarfPriests > 0 and members.nonDwarfPriests or members.healersPriestPrio, targetIndex).name

    local otHeal1 = assignMember(members, members.healers, targetIndex).name
    local otHeal2 = assignMember(members, members.healersPriestPrio, targetIndex).name
    local otHeal3 = assignMember(members, members.healersPriestPrio, targetIndex).name
    local otHeal4 = assignMember(members, members.healersDruidPrio, targetIndex).name

    addonData.sendChatMessage("Tank: " .. assignMember(members, members.tanks, targetIndex).name)
    addonData.sendChatMessage("Heal: " .. mtHeal1 .. " - " .. mtHeal2 .. " - " .. mtHeal3 .. " - " .. mtHeal4)
    
    addonData.sendChatMessage(" ")
    addonData.sendChatMessage("OT: " .. assignMember(members, members.tanks, targetIndex).name)
    addonData.sendChatMessage("OT/Melee-Heal: " .. otHeal1 .. " - " .. otHeal2 .. " - " .. otHeal3 .. " - " .. otHeal4)
    addonData.sendChatMessage(" ")
    addonData.sendChatMessage("Restliche Heiler: Range camp heal")
end

local function showTrashTechnicians(members)
    local technicianTank = assignMember(members, members.tanks, 1)
    local technicianHealer = assignMember(members, members.healers, 1)
    
    local targetIndex = 3
    local tanks = assignMembers(members, members.tanks, 2, targetIndex)
    tanks[#tanks].target = 7
    tanks[#tanks+1] = addonData.shallowCopy(tanks[#tanks])
    tanks[#tanks].target = 8
    local tankHealer = assignMembers(members, members.healers, 2, targetIndex)
    tankHealer[#tankHealer+1] = addonData.shallowCopy(tankHealer[#tankHealer])
    tankHealer[#tankHealer].target = 5
    local puller = assignMembers(members, members.hunters, 1, targetIndex)
    targetINdex = 4
    puller[#puller+1] = assignMember(members, members.rogues, targetIndex)
    targetINdex = 5
    puller[#puller+1] = assignMember(members, members.rogues, targetIndex)
    
    -- assign warlock tanks
    targetIndex = 1
    local warlocks = assignMembers(members, members.warlocks, 2, targetIndex)
    local warlockHealer = assignMembers(members, members.healers, 2, targetIndex)
    
    addonData.sendChatMessage("Techniker-Trash")
	addonData.sendChatMessage(" ")
    addonData.sendChatMessage("= TANK - HEAL - PULL/STUN =")
    addonData.sendChatMessage("Techniker: " .. technicianTank.name .. " - " .. technicianHealer.name)
    printAssigned(warlocks, warlockHealer)
    printAssigned(tanks, tankHealer, puller)
end

--------------------------------
-- AQ Targets
--------------------------------

local function showAQ(members)
	addonData.sendChatMessage("AQ standard targets")

    addonData.sendChatMessage(" ")
    addonData.sendChatMessage("= TANK - HEAL 1/2 =")
    local targetIndex = 1
    printAssigned(assignMembers(members, members.tanks, 3, targetIndex), assignMembers(members, members.healers, 3, targetIndex), assignMembers(members, members.healersPriestPrio, 3, targetIndex))
    targetIndex = 4
    printAssigned(assignMembers(members, members.tanks, 1, targetIndex), assignMembers(members, members.healers, 1, targetIndex), assignMembers(members, members.healersPriestPrio, 1, targetIndex))
end

local function showTrashAQ(members)
	addonData.sendChatMessage("AQ trash")
end

local function showSartura(members)
	addonData.sendChatMessage("Sartura")
    addonData.sendChatMessage("BUFF RAID: Amplify Magic")
    
    addonData.sendChatMessage(" ")
    addonData.sendChatMessage("Boss")
    addonData.sendChatMessage("= TANK 1/2 - HEAL 1/2 =")
    local targetIndex = 4
    printAssigned(assignMembers(members, members.tanks, 1, targetIndex), assignMembers(members, members.tanks, 1, targetIndex), assignMembers(members, members.healers, 1, targetIndex), assignMembers(members, members.healersPriestPrio, 1, targetIndex))
    
    -- "waste two" healers so healer assignments dont change
    -- waste first only if not enough paladins for first add stun
    if #members.healPaladins == 0 then
        assignMembers(members, members.healers, 1, targetIndex)
    end
    assignMembers(members, members.healersPriestPrio, 1, targetIndex)
    
    addonData.sendChatMessage(" ")
    addonData.sendChatMessage("Adds")
    addonData.sendChatMessage("= TANK - HEAL/STUN - STUN =")
    
    targetIndex = 1
    printAssigned(assignPlaceholder("kein Tank", targetIndex), assignMembers(members, members.paladins, 1, targetIndex), assignMembers(members, members.rogues, 2, targetIndex))
    targetIndex = 2
    printAssigned(assignMembers(members, members.tanks, 2, targetIndex), assignMembers(members, members.healers, 2, targetIndex), assignMembers(members, members.stunners, 2, targetIndex))
end

local function showTwins(members)
    local targetIndex = 1
	addonData.sendChatMessage("Twins")

    local leftHeal1 = assignMember(members, members.healers, targetIndex).name
    local rightHeal1 = assignMember(members, members.healers, targetIndex).name
    local leftHeal2 = assignMember(members, members.healers, targetIndex).name
    local rightHeal2 = assignMember(members, members.healers, targetIndex).name
    local leftHeal3 = assignMember(members, members.healersPriestPrio, targetIndex).name
    local rightHeal3 = assignMember(members, members.healersPriestPrio, targetIndex).name
    local leftHeal4 = assignMember(members, members.healersDruidPrio, targetIndex).name
    local rightHeal4 = assignMember(members, members.healersDruidPrio, targetIndex).name

    addonData.sendChatMessage(" ")
    addonData.sendChatMessage("= Links =")
    addonData.sendChatMessage("Tank: " .. assignMember(members, members.tanks, targetIndex).name .. " - " .. assignMember(members, members.warlocks, targetIndex).name)
    addonData.sendChatMessage("Heal: " .. leftHeal1 .. " - " .. leftHeal2 .. " - " .. leftHeal3 .. " - " .. leftHeal4)
    
   
    addonData.sendChatMessage(" ")
    addonData.sendChatMessage("= Rechts =")
    addonData.sendChatMessage("Tank: " .. assignMember(members, members.tanks, targetIndex).name .. " - " .. assignMember(members, members.warlocks, targetIndex).name)
    addonData.sendChatMessage("Heal: " .. rightHeal1 .. " - " .. rightHeal2 .. " - " .. rightHeal3 .. " - " .. rightHeal4)
    
    addonData.sendChatMessage(" ")
    addonData.sendChatMessage("= Käfer-Tank = ")
    addonData.sendChatMessage(assignMember(members, members.tanks, targetIndex).name)
end

local function showTrashChampions(members)
	addonData.sendChatMessage("Trash (Qiraji Champion Pack)")

    addonData.sendChatMessage(" ")
    addonData.sendChatMessage("= TANK 1/2 - HEAL - TRANQ SHOT =")
    local targetIndex = 1
    printAssigned(assignMembers(members, members.tanks, 2, targetIndex), assignMembers(members, members.offtanks, 2, targetIndex), assignMembers(members, members.healers, 2, targetIndex), assignMembers(members, members.hunters, 2, targetIndex))
    
    targetIndex = 3
    local champTank = assignMembers(members, members.tanks, 1, targetIndex)
    local champHeal = assignMembers(members, members.healers, 1, targetIndex)
    local champPriest = assignMembers(members, #members.dwarfPriests > 0 and members.dwarfPriests or members.healersPriestPrio, 1, targetIndex)

    targetIndex = 4
    printAssigned(assignMembers(members, members.tanks, 2, targetIndex), assignMembers(members, members.offtanks, 2, targetIndex), assignMembers(members, members.healers, 2, targetIndex), assignMembers(members, members.hunters, 2, targetIndex))
    
    addonData.sendChatMessage(" ")
    addonData.sendChatMessage("= TANK 1/2 - HEAL 1/2 =")
    targetIndex = 5
    printAssigned(champTank, assignMembers(members, members.tanks, 1, targetIndex), champHeal, champPriest)
    addonData.sendChatMessage(champPriest[1].name .. ": dispell mob und fearward tank")
end

--------------------------------
-- Nax Targets
--------------------------------

local function showNax(members)
    addonData.sendChatMessage("Naxx standard targets")

    addonData.sendChatMessage(" ")
    addonData.sendChatMessage("= TANK - HEAL 1/2/3 =")
    local targetIndex = 1
    printAssigned(assignMembers(members, members.tanks, 1, targetIndex), assignMembers(members, members.healers, 1, targetIndex), assignMembers(members, members.healers, 1, targetIndex), assignMembers(members, members.healersPriestPrio, 1, targetIndex))
    local targetIndex = 2
    printAssigned(assignMembers(members, members.tanks, 2, targetIndex), assignMembers(members, members.healers, 2, targetIndex), assignMembers(members, members.healersPriestPrio, 2, targetIndex))
    targetIndex = 4
    printAssigned(assignMembers(members, members.tanks, 1, targetIndex), assignMembers(members, members.healers, 1, targetIndex), assignMembers(members, members.healersPriestPrio, 1, targetIndex))
end

local function showTrashPatchwerk(members)
	addonData.sendChatMessage("Patchwerk Trash")
    addonData.sendChatMessage(" ")
    addonData.sendChatMessage("Kick Scientist Heal!")
    addonData.sendChatMessage(" ")
    addonData.sendChatMessage("= TANK - HEAL =")
    local targetIndex = 1
    printAssigned(assignMembers(members, members.tanks, 1, targetIndex), assignMembers(members, members.healers, 1, targetIndex), assignMembers(members, members.healers, 1, targetIndex), assignMembers(members, members.healersPriestPrio, 1, targetIndex))
    targetIndex = 2
    printAssigned(assignMembers(members, members.tanks, 4, targetIndex), assignMembers(members, members.healers, 4, targetIndex))
end

local function showPatchwerk(members)
	addonData.sendChatMessage("Patchwerk")
    addonData.sendChatMessage(" ")   
    addonData.sendChatMessage("BUFF TANKS: Amplify Magic")
    addonData.sendChatMessage("BUFF SOAKER: LoH")
    addonData.sendChatMessage(" ") 
    addonData.sendChatMessage(addonData.colorize("DEBUFFS +:", "00FF00") .. " Screech, Murmeln")
    addonData.sendChatMessage(addonData.colorize("DEBUFFS -:", "FF0000") .. " Hunter's Mark, Mindflay")
    addonData.sendChatMessage(" ")    
    addonData.sendChatMessage("= TANK - HEAL 1/2/3 =")
    local targetIndex = 1
    printAssigned(assignMembers(members, members.tanks, 1, targetIndex), assignMembers(members, members.healers, 1, targetIndex), assignMembers(members, members.healersPriestPrio, 1, targetIndex), assignMembers(members, members.healersDruidPrio, 1, targetIndex))
    targetIndex = "Soaker"
    printAssigned(assignMembers(members, members.tanks, 3, targetIndex), assignMembers(members, members.healers, 3, targetIndex), assignMembers(members, members.healersPriestPrio, 3, targetIndex), assignMembers(members, members.healers, 3, targetIndex))
end

local function showGluth(members)
	addonData.sendChatMessage("Gluth")
    addonData.sendChatMessage(" ")    
    local targetIndex = 1
    local tank1 = assignMember(members, members.tanks, targetIndex).name
    local tank2 = assignMember(members, members.tanks, targetIndex).name
    local heal1 = assignMember(members, members.healers, 1, targetIndex).name
    local heal2IsDwarf = #members.dwarfPriests > 0
    local heal2 = assignMember(members, heal2IsDwarf and members.dwarfPriests or members.healersPriestPrio, targetIndex).name
    local heal3IsDwarf = #members.dwarfPriests > 0
    local heal3 = assignMember(members, heal3IsDwarf and members.dwarfPriests or members.healersPriestPrio, targetIndex).name
    local heal4IsDwarf = #members.dwarfPriests > 0
    local heal4 = assignMember(members, heal4IsDwarf and members.dwarfPriests or members.healersDruidPrio, targetIndex).name
    
    addonData.sendChatMessage(" ")
    addonData.sendChatMessage("Tank: " .. tank1 .. " - " .. tank2)
    addonData.sendChatMessage("Heal: " .. heal1 .. " - " .. heal2 .. " - " .. heal3 .. " - " .. heal4)
    addonData.sendChatMessage(" ")
    addonData.sendChatMessage("Fearward:")
    addonData.sendChatMessage(tank1 .. ": " .. (heal2IsDwarf and heal2 or "n/a") .. " - " .. (heal3IsDwarf and heal3 or "n/a"))
    addonData.sendChatMessage(tank2 .. ": " .. (heal4IsDwarf and heal4 or "n/a"))
end

local function showThaddius(members)
	addonData.sendChatMessage("Thaddius")
    addonData.sendChatMessage(" ")    
    local targetIndex = 1
    
    addonData.sendChatMessage("Aufstellung: " .. addonData.colorize("- - - -", "FF0000") .. " Thaddius " .. addonData.colorize("+ + + +", "40C7EB"))
    addonData.sendChatMessage(" ")
    
    local leftHeal = assignMember(members, members.healers, targetIndex).name
    local rightHeal = assignMember(members, members.healers, targetIndex).name
    
    while (#members.healers > 0) do
        leftHeal = leftHeal .. " - " .. assignMember(members, members.healers, targetIndex).name
        if #members.healers > 0 then
            rightHeal = rightHeal .. " - " .. assignMember(members, members.healers, targetIndex).name
        end
    end
    
    addonData.sendChatMessage("= Links =")
    addonData.sendChatMessage("Tank: " .. assignMember(members, members.tanks, targetIndex).name .. " - " .. assignMember(members, members.tanks, targetIndex).name)
    addonData.sendChatMessage("Heal: " .. leftHeal)
    addonData.sendChatMessage(" ")
    addonData.sendChatMessage("= Rechts =")
    addonData.sendChatMessage("Tank: " .. assignMember(members, members.tanks, targetIndex).name .. " - " .. assignMember(members, members.tanks, targetIndex).name)
    addonData.sendChatMessage("Heal: " .. rightHeal)
end

local function showTrashSpider(members)
	addonData.sendChatMessage("Spiderwing Trash")
    addonData.sendChatMessage(" ")  
    addonData.sendChatMessage("DECURSE (Prio tanks)")
    addonData.sendChatMessage("CLEANSE POISON nach charge")     
    addonData.sendChatMessage(" ")   
    local targetIndex = 1
    addonData.sendChatMessage("= TANK - HEAL =")  
    printAssigned(assignMembers(members, members.tanks, 6, targetIndex), assignMembers(members, members.healers, 6, targetIndex))
end

local function showTrashFaerlina(members)
    addonData.sendChatMessage("Faerlina Trash") 
    addonData.sendChatMessage(" ")
    addonData.sendChatMessage("BUFF RAID: Schattenschutz")

    local targetIndex = 2
    addonData.sendChatMessage(" ")
    addonData.sendChatMessage("= Kick 1/2 - Curse of Tongues =")
    local numWarlocks = #members.warlocks
    warlocks = assignMembers(members, members.warlocks, 5, targetIndex)
    local i = 1
    while numWarlocks < 5 do
        numWarlocks = numWarlocks + 1
        warlocks[numWarlocks].name = warlocks[i].name
        i = i + 1
    end
    
    printAssigned(assignMembers(members, members.interrupters, 5, targetIndex), assignMembers(members, members.interrupters, 5, targetIndex), warlocks)
end

local function showFaerlina(members)
    addonData.sendChatMessage("Faerlina")
    addonData.sendChatMessage(" ") 
    addonData.sendChatMessage("PREPOT: Greater Fire Protection")
    addonData.sendChatMessage(" ") 
    printAssigned(assignMembers(members, members.tanks, 1, "Boss"), assignMembers(members, members.healers, 1, "Boss"), assignMembers(members, members.healers, 1, "Boss"), assignMembers(members, members.healersPriestPrio, 1, "Boss"))
    local targetIndex = 3
    local tank1 = { assignMemberDontRemove(members, members.tanks, targetIndex), assignMember(members, members.tanks, targetIndex +1) }
    targetIndex = 5
    local tank2 = { assignMemberDontRemove(members, members.tanks, targetIndex), assignMember(members, members.tanks, targetIndex +1) }
    targetIndex = 1
    local focusTanks = assignMembers(members, members.tanks, 2, targetIndex)
    
    addonData.sendChatMessage(" ")
    printAssigned(focusTanks)
    printAssigned(tank1, assignMembers(members, members.healers, 1, targetIndex))
    printAssigned(tank2, assignMembers(members, members.healers, 1, targetIndex))
    addonData.sendChatMessage(" ")
    printAssigned(assignMembers(members, members.mindControllers, 1, "Mindcontrol"), assignMembers(members, members.mindControllers, 1, "Mindcontrol"))
    
end

local function showRazuvious(members)
    addonData.sendChatMessage("Razuvious")
    addonData.sendChatMessage(" ")
    addonData.sendChatMessage("BUFF ADDS: Amplify Magic") 
    addonData.sendChatMessage(" ")
    
    addonData.sendChatMessage("= MINDCONTROL 1/2 =")  
    local targetIndex = 3
    printAssigned(assignMembers(members, members.mindControllers, 1, targetIndex), assignMembers(members, members.mindControllers, 1, targetIndex))
    
    addonData.sendChatMessage(" ")
    addonData.sendChatMessage("= TANK =")  
    local targetIndex = 4
    addonData.sendChatMessage("Kein Sunder Armor, kein TF!") 
    printAssigned(assignMembers(members, members.tanks, 3, targetIndex))
end

local function showNoth(members)
    addonData.sendChatMessage("Noth")
    addonData.sendChatMessage(" ")
    addonData.sendChatMessage("DECURSE raid!")
    addonData.sendChatMessage("DISPELL prio Maintank!")
    addonData.sendChatMessage(" ")
    addonData.sendChatMessage("= TANK - HEAL 1/2/3 =")  
    printAssigned(assignMembers(members, members.tanks, 1, "Noth"), assignMembers(members, members.healers, 1, "Noth"), assignMembers(members, members.healers, 1, "Noth"), assignMembers(members, members.healersPriestPrio, 1, "Noth"))
    addonData.sendChatMessage(" ")
    addonData.sendChatMessage("= TANK - HEAL - DD 1/2 =")  
    printAssigned(assignMembers(members, members.tanks, 1, "NW"), assignMembers(members, members.healers, 1, "NW"), assignMembers(members, members.hunters, 1, "NW"), assignMembers(members, members.warlocks, 1, "NW"))
    printAssigned(assignMembers(members, members.tanks, 1, "NO"), assignMembers(members, members.healers, 1, "NO"), assignMembers(members, members.hunters, 1, "NO"), assignMembers(members, members.warlocks, 1, "NO"))
    printAssigned(assignMembers(members, members.tanks, 1, "SW"), assignMembers(members, members.healers, 1, "SW"), assignMembers(members, members.hunters, 1, "SW"), assignMembers(members, members.warlocks, 1, "SW"))
end

local function showLoatheb(members)
    addonData.sendChatMessage("Loatheb")
    addonData.sendChatMessage(" ")
    addonData.sendChatMessage("BUFF WARRIORS: Amplify Magic") 
    addonData.sendChatMessage("PREPOT: Greater Shadow Protection")
    addonData.sendChatMessage("SPORE GROUPS: 6 - 5 - 4 - 3 - 2")
    addonData.sendChatMessage(" ")
    printAssigned(assignMembers(members, members.tanks, 1, "MT"))
    
    printAssigned(assignMembers(members, #members.paladins > 0 and members.paladins or members.healersAlternating, 1, "1"))
    printAssigned(assignMembers(members, #members.paladins > 0 and members.paladins or members.healersAlternating, 1, "2"))
    for i=1, min(#members.healersAlternating, 11) do
        printAssigned(assignMembers(members, members.healersAlternating, 1, tostring(i+2)))
    end
    addonData.sendChatMessage(" ")
    addonData.sendChatMessage("PrePot GSPP")
    addonData.sendChatMessage("2:10 GSPP")
    addonData.sendChatMessage("2:40 -")
    addonData.sendChatMessage("3:10 Bandage")
    addonData.sendChatMessage("3:40 Healthstone")
    addonData.sendChatMessage("4:10 GSPP & Bandage")
    addonData.sendChatMessage("4:40 -")
    addonData.sendChatMessage("5:10 Bandage")
    addonData.sendChatMessage("5:25 alles")
end

local function showGothik(members)
    addonData.sendChatMessage("Gothik")
    addonData.sendChatMessage(" ")
    
    local casterCount = #members.warlocks + #members.mages + #members.shadowPriests
    local physicalCount = #members.hunters + #members.rogues + #members.feralDruids + #members.warriors - 6
    
    local numRoguesToUD = (casterCount + physicalCount)/2 - casterCount
    
    local udTank1 = assignMembers(members, members.tanks, 1, "Tanks")
    local livingTank1 = assignMembers(members, members.tanks, 1, "Tanks")
    local livingTank2 = assignMembers(members, members.tanks, 1, "Tanks")
    local livingTank3 = assignMembers(members, members.tanks, 1, "Tanks")
    local udTank2 = assignMembers(members, members.tanks, 1, "Tanks")
    local udTank3 = assignMembers(members, members.tanks, 1, "Tanks")
    
    addonData.sendChatMessage("= Living Seite =")
    printAssigned(livingTank1, livingTank2, livingTank3)       
    
    local healer = "Healer"
    
    healer = healer .. " - " .. assignMember(members, members.healPaladins, "Healer").name    
    for i=1,(#members.healers/2)-1 do
        healer = healer .. " - " .. assignMember(members, #members.healPriests > 1 and members.healPriests or members.healDruids, "Healer").name
    end
    addonData.sendChatMessage(healer)

    local dds = "DD - ALLE CASTER"
    if numRoguesToUD > 0 then
        for i=1,numRoguesToUD do
            dds = dds .. " - " .. assignMember(members, members.rogues, "Melees").name
        end
    end
    addonData.sendChatMessage(dds)
    
    addonData.sendChatMessage(" ")
    addonData.sendChatMessage("= Undead Seite =")
    printAssigned(udTank1, udTank2, udTank3)
    addonData.sendChatMessage("Alle restlichen Spieler")
end

local function showFourHorsemen(members)
    addonData.sendChatMessage("Four Horsemen")
    addonData.sendChatMessage(" ")
    addonData.sendChatMessage("MELEE PREPOT: Greater Fire Protection")
    addonData.sendChatMessage(" ")
    
    local tank1Thane = assignMembers(members, members.tanks, 1, "Tank")
    local tank2Thane = assignMembers(members, members.tanks, 1, "Tank")
    
    addonData.sendChatMessage("North - Highlord Mograine - " .. assignMember(members, members.hunters, "Hunter").name)
    printAssigned(assignMembers(members, members.tanks, 1, "Tank"), assignMembers(members, members.tanks, 1, "Tank"))
    addonData.sendChatMessage("1/4/7 - " .. assignMember(members, members.healersPaladinPrio, "Healer").name)
    addonData.sendChatMessage("2/5/8 - " .. assignMember(members, members.healersPriestPrio, "Healer").name)
    addonData.sendChatMessage("3/6/9 - " .. assignMember(members, members.healersDruidPrio, "Healer").name)
    addonData.sendChatMessage(" ")
    
    addonData.sendChatMessage("East - Thane Korth'azz - " .. assignMember(members, members.hunters, "Hunter").name)
    printAssigned(tank1Thane, tank2Thane)
    addonData.sendChatMessage("1/4/7 - " .. assignMember(members, members.healersPaladinPrio, "Healer").name)
    addonData.sendChatMessage("2/5/8 - " .. assignMember(members, members.healersPriestPrio, "Healer").name)
    addonData.sendChatMessage("3/6/9 - " .. assignMember(members, members.healersDruidPrio, "Healer").name)
    addonData.sendChatMessage(" ")
    
    addonData.sendChatMessage("South - Lady Blaumeux - " .. assignMember(members, members.hunters, "Hunter").name)
    printAssigned(assignMembers(members, members.tanks, 1, "Tank"), assignMembers(members, members.tanks, 1, "Tanks"))
    addonData.sendChatMessage("1/4/7 - " .. assignMember(members, members.healersPaladinPrio, "Healer").name)
    addonData.sendChatMessage("2/5/8 - " .. assignMember(members, members.healersPriestPrio, "Healer").name)
    addonData.sendChatMessage("3/6/9 - " .. assignMember(members, members.healersDruidPrio, "Healer").name)
    addonData.sendChatMessage(" ")
    
    addonData.sendChatMessage("West - Sir Zeliek - " .. assignMember(members, members.hunters, "Hunter").name)
    printAssigned(assignMembers(members, members.tanks, 1, "Tank"), assignMembers(members, members.tanks, 1, "Tanks"))
    addonData.sendChatMessage("1/4/7 - " .. assignMember(members, members.healersPaladinPrio, "Healer").name)
    addonData.sendChatMessage("2/5/8 - " .. assignMember(members, members.healersPriestPrio, "Healer").name)
    addonData.sendChatMessage("3/6/9 - " .. assignMember(members, members.healersDruidPrio, "Healer").name)
    addonData.sendChatMessage(" ")
    

    
    addonData.sendChatMessage("DD:")
    addonData.sendChatMessage("gegen Uhrzeigersinn")
    addonData.sendChatMessage("Gruppen 2 & 3 Thane Korth'azz")
    addonData.sendChatMessage("Gruppen 4 & 5 Safeposition")
    addonData.sendChatMessage("Gruppen 6 & 7 Highlord Mograine")
end

local function showSapphiron(members)
    addonData.sendChatMessage("Sapphiron")
    addonData.sendChatMessage(" ")
    addonData.sendChatMessage("PREPOT: Greater Frost Protection")
    addonData.sendChatMessage("BUFF RAID: Amplify Magic")
    addonData.sendChatMessage("BUFF MT: LoH")
    addonData.sendChatMessage("DECURSE")
    addonData.sendChatMessage(" ")
    addonData.sendChatMessage("Groups 1-4: North")
    addonData.sendChatMessage("Groups 5-8: South")
    addonData.sendChatMessage(" ")
    addonData.sendChatMessage("= TANK - HEAL 1/2/3 =")
    printAssigned(assignMembers(members, members.tanks, 1, "Boss"), assignMembers(members, members.healPaladins, 1, "Boss"), assignMembers(members, members.healPaladins, 1, "Boss"), assignMembers(members, members.healDruids, 1, "Boss"))
    addonData.sendChatMessage(" ")
    addonData.sendChatMessage("= GROUP - HEAL =")
    printAssigned(assignMembers(members, members.healersPriestPrio, 1, "1"))
    printAssigned(assignMembers(members, members.healersPriestPrio, 1, "2"))
    printAssigned(assignMembers(members, members.healersPaladinPrio, 1, "3"))
    printAssigned(assignMembers(members, #members.healPaladins > 1 and members.healersPaladinPrio or members.healersPriestPrio, 1, "4"))
    printAssigned(assignMembers(members, members.healersPriestPrio, 1, "5"))
    printAssigned(assignMembers(members, members.healersPriestPrio, 1, "6"))
    printAssigned(assignMembers(members, members.healersPaladinPrio, 1, "7"))
    printAssigned(assignMembers(members, members.healersPaladinPrio, 1, "8"))
end

local function showKelthuzad(members)
    addonData.sendChatMessage("Kel'Thuzad")
    addonData.sendChatMessage(" ")
    addonData.sendChatMessage("PREPOT: Greater Frost Protection")
    addonData.sendChatMessage("P3: 3. und 4. Tank markieren und tanken ein Add")
    
    addonData.sendChatMessage(" ")
    addonData.sendChatMessage("= TANK - HEAL 1/2/3 =")
    local targetIndex = 1
    printAssigned(assignMembers(members, members.tanks, 4, targetIndex), assignMembers(members, members.healers, 1, targetIndex), assignMembers(members, members.healers, 1, targetIndex), assignMembers(members, members.healersPriestPrio, 1, targetIndex))
    
    addonData.sendChatMessage(" ")
    addonData.sendChatMessage("= KICK =")
    addonData.sendChatMessage("Gruppe 3: {rt3}")
    addonData.sendChatMessage("Gruppe 4: {rt2}")
    addonData.sendChatMessage("Gruppe 5: {rt1}")
end

--------------------------------
-- Interface
--------------------------------

local function showDefaultTargets(members)
    if addonData.isBWL() then
        showBWL(members)
    elseif addonData.isMC() then
        showMC(members)
    elseif addonData.isAQ() then
        showAQ(members)
    elseif addonData.isNax() then
        showNax(members)
    else
        addonData.printColor("Es gibt keine speziellen Trash Targets für deinen Ort bzw. dein selektiertes Ziel.")
    end
end

local function showTargets(debugTarget)
    local members = addonData.getRaidMembersByRole()
	
	local targetGuid = UnitGUID("target")
	if targetGuid and (debugTarget == nil or debugTarget == "") then
		local npcId = addonData.getNpcId(targetGuid)
		
        --MC
        local domo = 12018
		local sulfuron = 12098
        
        --BWL
		local grethok = 12557
        local vael = 13020
        local firemaw = 11983
 
        local deathTalonPack = { 12467, 12464, 12465, 12463 }
        local technicianPack = { 13996, 12459, 12461, 12457 }
        
        --AQ
        local twins = { 15276, 15275 }
        local sartura = 15516
        local trashAQ = { 15233, 15247 } --guardians, bug trio trash
        local championPack = { 15252, 15250, 15246 }
    
        -- Nax
        local patchwerkTrash = { 16020, 16021, 16022 }
        local faerlinaTrash = { 15980, 15981 }
        local spiderTrash = {15975, 15974, 15976 }
        local patchwerk = 16028
        local gluth = 15932
        local thaddius = {15929, 15930, 15928}
        local faerlina = 15953
        local razuvious = 16061
        local noth = 15954
        local loatheb = 16011
        local gothik = 16060
        local horsemen = { 16065, 16063, 16064, 16062 }
        local sapphiron = 15989
        local kelthuzad = 15990
    
	    if npcId == domo then
			showDomo(members)
		elseif npcId == sulfuron then
			showSulfuron(members)
		elseif npcId == grethok then
			showRazorgore(members)
        elseif addonData.hasValue(deathTalonPack, npcId) then
            showTrashVael(members)
        elseif addonData.hasValue(technicianPack, npcId) then
            showTrashTechnicians(members)
        elseif npcId == firemaw then
            showFiremaw(members)
        elseif addonData.hasValue(twins, npcId) then
            showTwins(members)
        elseif npcId == sartura then
            showSartura(members)
        elseif addonData.hasValue(championPack, npcId) then
            showTrashChampions(members)
        elseif addonData.hasValue(patchwerkTrash, npcId) then
            showTrashPatchwerk(members)
        elseif addonData.hasValue(faerlinaTrash, npcId) then
            showTrashFaerlina(members)
        elseif addonData.hasValue(spiderTrash, npcId) then
            showTrashSpider(members)
        elseif npcId == patchwerk then
            showPatchwerk(members)
        elseif npcId == gluth then
            showGluth(members)
        elseif addonData.hasValue(thaddius, npcId) then
            showThaddius(members)
        elseif npcId == faerlina then
            showFaerlina(members)
        elseif npcId == razuvious then
            showRazuvious(members)
        elseif npcId == noth then
            showNoth(members)
        elseif npcId == loatheb then
            showLoatheb(members)
        elseif npcId == gothik then
            showGothik(members)
        elseif addonData.hasValue(horsemen, npcId) then
            showFourHorsemen(members)
        elseif npcId == sapphiron then
            showSapphiron(members)
        elseif npcId == kelthuzad then
            showKelthuzad(members)
		else
			showDefaultTargets(members)
		end
	else
        if debugTarget == nil or debugTarget == "" then
            showDefaultTargets(members)
        else
            if debugTarget == "mc" then
                showMC(members)
            elseif debugTarget == "bwl" then
                showBWL(members)
            elseif debugTarget == "aq" then
                showAQ(members)
            elseif debugTarget == "nax" then
                showNax(members)
            elseif debugTarget == "domo" then
                showDomo(members)
            elseif debugTarget == "sulfuron" then
                showSulfuron(members)
            elseif debugTarget == "razor" then
                showRazorgore(members)
            elseif debugTarget == "vael" then
                showTrashVael(members)
            elseif debugTarget == "firemaw" then
                showFiremaw(members)
            elseif debugTarget == "technicians" then
                showTrashTechnicians(members)
            elseif debugTarget == "sartura" then
                showSartura(members)
            elseif debugTarget == "twins" then
                showTwins(members)
            elseif debugTarget == "champions" then
                showTrashChampions(members)
            elseif debugTarget == "trashpatchwerk" then
                showTrashPatchwerk(members)
            elseif debugTarget == "trashfaerlina" then
                showTrashFaerlina(members)
            elseif debugTarget == "trashspider" then
                showTrashSpider(members)
            elseif debugTarget == "patchwerk" then
                showPatchwerk(members)
            elseif debugTarget == "gluth" then
                showGluth(members)
            elseif debugTarget == "thaddius" then
                showThaddius(members)
            elseif debugTarget == "faerlina" then
                showFaerlina(members)
            elseif debugTarget == "razuvious" then
                showRazuvious(members)
            elseif debugTarget == "noth" then
                showNoth(members)
            elseif debugTarget == "loatheb" then
                showLoatheb(members)
            elseif debugTarget == "gothik" then
                showGothik(members)
            elseif debugTarget == "horsemen" then
                showFourHorsemen(members)
            elseif debugTarget == "sapphiron" then
                showSapphiron(members)
            elseif debugTarget == "kelthuzad" then
                showKelthuzad(members)
            end
		end
	end
end

addonData.showTargets = showTargets